/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/dct/dct.hh"
#include "libvideocoding/dct/fdct_fp_aan.hh"
#include "libvideocoding/dct/idct_int_aan.hh"
#include "config.h"

#if ENABLE_MMX
#include "libvideocoding/dct/fdct_mmx.hh"
#include "libvideocoding/dct/idct_mmx.hh"
#endif

#include <libvideogfx/arch/cpu.hh>


namespace videocoding {
  using namespace videogfx;

  FDCTBlk8x8<Pixel>* GetFDCT8x8_8(bool accurate)
  {
    CPU_Capabilities* cpu = CPU_Capabilities::AskCapabilities();

#if ENABLE_MMX
    if (cpu->AskArchitecture()==CPUArch_X86 && cpu->HasMMX())
      {
	return new FDCTBlk_AccurateMMX8;
      }
#endif

    return new FDCTBlk_AccurateFP<Pixel>;
  }


  FDCTBlk8x8<int16>* GetFDCT8x8_16(bool accurate)
  {
    CPU_Capabilities* cpu = CPU_Capabilities::AskCapabilities();

#if ENABLE_MMX
    if (cpu->AskArchitecture()==CPUArch_X86 && cpu->HasMMX())
      {
	return new FDCTBlk_AccurateMMX16;
      }
#endif

    return new FDCTBlk_AccurateFP<int16>;
  }


  IDCTBlk8x8<Pixel>* GetIDCT8x8_8(bool ieee_compliant)
  {
    CPU_Capabilities* cpu = CPU_Capabilities::AskCapabilities();

#if ENABLE_MMX
    if (cpu->AskArchitecture()==CPUArch_X86 && cpu->HasMMX())
      {
	return new IDCTBlk_MMX8;
      }
#endif

    return new IDCTBlk_Integer<Pixel>(0,255);
  }


  IDCTBlk8x8<int16>* GetIDCT8x8_16(bool ieee_compliant)
  {
    return new IDCTBlk_Integer<int16>(-255,255);
  }

}
