/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/decoder/video.hh,v 1.7 2002/05/02 17:24:58 farin Exp $

    MPEG-2 video decoder
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_DECODER_VIDEO_HH
#define LIBVIDEOCODING_MPEG2_DECODER_VIDEO_HH

#include <libvideocoding/mpeg2/syntax/macroblock.hh>
#include <libvideocoding/mpeg2/decoder/motioncomp.hh>
#include <libvideocoding/dct/dct.hh>
#include <libvideogfx/audio/fileio/timedsink.hh>


namespace videocoding {
  using namespace videogfx;

  class MP2_ImageSink : public TimedPresentationSink
  {
  public:
    virtual ~MP2_ImageSink() { }

    virtual bool NeedPictureHeader() const { return false; }
    virtual bool NeedPictureData()   const { return false; }

    virtual void SendImage(const Image<Pixel>&,
			   const MP2_PictureHeader* first_ph,
			   const MP2_PictureHeader* second_ph,
			   const Bitmap<MP2_Macroblock>* first_pd,
			   const Bitmap<MP2_Macroblock>* second_pd) = 0;
  };


  class MP2_VideoDecoder : public MP2_MacroblockSink
  {
  public:
    MP2_VideoDecoder();
    ~MP2_VideoDecoder();

    void Reset();
    void SetSink(MP2_ImageSink& s) { d_sink=&s; }

    MP2_Macroblock* RequestMacroblock(int mbx,int mby);

    void SendSequenceHeader(const MP2_SequenceHeader&);
    void SendPictureHeader (const MP2_PictureHeader&);
    void SendMacroblock(MP2_Macroblock*,int mbx,int mby);

    void FlushAllPics();

  private:
    MP2_Macroblock mbmem;
    MP2_ImageSink* d_sink;

    const MP2_SequenceHeader* d_seqhdr;

    Image<Pixel>  d_img[3];

    //Image<Pixel>* d_destimg;     // == NULL after a picture has been sent to the sink
    bool          d_have_top;    // pichdr for top field has arrived
    bool          d_have_bottom; // pichdr for bottom field has arrived
    int           d_pic_mem;     // 0 for current picture, 1 for next picture
    int           d_field_idx;   // 0 for first field in picture, 1 for second field


    // extra data for postprocessor stages

    bool  d_need_picdata;

    MP2_PictureHeader  d_pichdr[2][2];  //  [ CURRENT/NEXT ] [ FIRST-FIELD/SECOND-FIELD ]
    //const MP2_PictureHeader* d_pichdr;   // d_pichdr[d_field_idx] is current picture header

    Bitmap<MP2_Macroblock>  d_picdata[2][2];

    bool                    d_data_valid[2][2];


    // accelerated algorithms

    IDCTBlk8x8<Pixel>*  d_idct;
    MP2_MotionCompensation d_motioncomp;


    void FlushFrame(int mem);
    void FlushField(int mem,bool top);
  };

}

#endif
