/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/encoder/quantize.hh,v 1.4 2002/05/12 21:41:47 farin Exp $

    Quantization functions for single DCT blocks.
 ********************************************************************************
    Copyright (C) 2000 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_ENCODER_QUANTIZE_HH
#define LIBVIDEOCODING_MPEG2_ENCODER_QUANTIZE_HH

#include <libvideocoding/datatypes/dct.hh>
#include <libvideocoding/mpeg2/syntax/macroblock.hh>


namespace videocoding {

  class MP2_Quantizer
  {
  public:
    virtual ~MP2_Quantizer() { }
    virtual const char* AskName() const { return "(unnamed)"; }


    // initialization ; all of these initialization routines have to be called before the object is used

    virtual void SetIntraQMatrix(const QMatrix_8x8&) = 0;
    virtual void SetInterQMatrix(const QMatrix_8x8&) = 0;
    virtual void SetDCPrecision(int prec) = 0;
    virtual void SetMPEG2(bool flag) = 0;

    /* The input and output DCT-blocks may be the same.
       The methods returning a bool value return true if the block contains non-zero coefficients. */

    virtual void Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale) = 0;
    virtual void Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale) = 0;

    virtual bool Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale, int32& out_of_range) = 0;
    virtual bool Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale, int32& out_of_range) = 0;

    virtual void Dequantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale) = 0;
    virtual void Dequantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in, int qscale) = 0;

    void QuantizeMB  (MP2_Macroblock& out,const MP2_Macroblock& in);
    void QuantizeMB  (MP2_Macroblock& out,const MP2_Macroblock& in,int32& out_of_range,int16& cbp);
    void DequantizeMB(MP2_Macroblock& out,const MP2_Macroblock& in);
  };


  MP2_Quantizer* GetQuantizer(bool accurate=true);

}

#endif
