/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/mpeg2/encoder/quantize_mmx.hh"
#include "libvideocoding/mpeg2/syntax/video.hh"

#include <iostream>

namespace videocoding {
  using namespace std;


  MP2_Quantizer_MMX::MP2_Quantizer_MMX()
  {
    d_high_precision = false;
    SetDCPrecision(8);
    SetIntraQMatrix(MP2_DefaultQuantMatrix_Intra);
    SetInterQMatrix(MP2_DefaultQuantMatrix_Inter);

    for (int q=1;q<=112;q++)
      {
	uint64 q4 = q;
	q4 |= q4 << 16;
	q4 |= q4 << 32;

	qscale_4times[q] = q4;
      }
  }


  void MP2_Quantizer_MMX::SetDCPrecision(int prec)
  {
    d_dc_precision = prec;

    int preci = prec -= 8;
    int dcoffset = 4>>preci;

    for (int q=1;q<=112;q++)
      {
	offs_intra[q][0][0] = dcoffset;
	qmat_intra[q][0][0] = (1<<16) >> (3-preci);
	if (preci==3) qmat_intra[q][0][0] = (1<<16)-1;
      }

    d_dc_shift = 3-preci;
  }


  void MP2_Quantizer_MMX::SetIntraQMatrix(const QMatrix_8x8& m)
  {
    d_intra = m;


    for (int q=1;q<=112;q++)
      for (int v=0;v<8;v++)
	for (int u=0;u<8;u++)
	  {
	    if (u==0 && v==0)
	      continue;

	    int val;

	    val = 16*((1<<16)-1)/(d_intra.mat[v][u]*q);
	    assert(val < (1<<16));
	    qmat_intra[q][v][u] = val;


	    val = (d_intra.mat[v][u]*q)/32;  // /16/2
	    offs_intra[q][v][u] = val;
	  }
  }


  void MP2_Quantizer_MMX::SetInterQMatrix(const QMatrix_8x8& m)
  {
    d_inter = m;

    for (int q=1;q<=112;q++)
      for (int v=0;v<8;v++)
	for (int u=0;u<8;u++)
	  {
	    int val;

	    val = 16*((1<<16)-1)/(d_inter.mat[v][u]*q);
	    assert(val < (1<<16));
	    qmat_inter[q][v][u] = val;
	  }
  }


  void MP2_Quantizer_MMX::Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  {
    // MP2_Quantize_Intra(in,out,d_intra,qscale,d_dc_precision);  return;

    const uint16* qtab = &qmat_intra[qscale][0][0];
    const uint16* otab = &offs_intra[qscale][0][0];
    const int16* inp  = &in.coeff[0][0];
    int16* outp = &out.coeff[0][0];

#if 0
    int16 copy[8][8];

    for (int i=0;i<64;i++)
      copy[0][i] = in.coeff[0][i];
#endif


    if (0) // d_high_precision)   TODO: high precision code needs further testing...
      {
	for (int i=0;i<64/4;i++)
	  {
	    __asm__
	      (
	       // ---- calc abs(in)

	       "movq     (%0),%%mm1\n\t"  // signed input values
	       "pxor     %%mm7,%%mm7\n\t"
	       "pcmpgtw  %%mm1,%%mm7\n\t" // mm7: <0 masks (0xFFFF iff in<0)

	       "movq     %%mm1,%%mm0\n\t"
	       "psllw    $1,%%mm0\n\t"
	       "pand     %%mm7,%%mm0\n\t"
	       "psubw    %%mm0,%%mm1\n\t" // mm1=abs(in)

	       // ---- quantize

	       "paddw    (%3),%%mm1\n\t"   // intra quantizer offset

#if 1
	       "movq     %%mm1,%%mm2\n\t"
	       "pmullw   (%2),%%mm2\n\t"

	       "movq     (%2),%%mm3\n\t"
	       "psrlw    $3,%%mm3\n\t"
	       "psrlw    $1,%%mm2\n\t"
	       "paddw    %%mm3,%%mm2\n\t"
	       "psrlw    $15,%%mm2\n\t"
#endif

	       "pmulhw   (%2),%%mm1\n\t"
	       "paddw    %%mm2,%%mm1\n\t"


	       // ---- restore sign

	       "movq     %%mm1,%%mm2\n\t"
	       "psllw    $1,%%mm2\n\t"
	       "pand     %%mm7,%%mm2\n\t"
	       "psubw    %%mm2,%%mm1\n\t"

	       "movq     %%mm1,(%1)\n\t"

	       : : "r"(inp),"r"(outp),"r"(qtab),"r"(otab)
	       );

	    inp += 4;
	    outp += 4;
	    qtab += 4;
	    otab += 4;
	  }
      }
    else
      {
	for (int i=0;i<64/4;i++)
	  {
	    __asm__
	      (
	       // ---- calc abs(in)

	       "movq     (%0),%%mm1\n\t"  // signed input values
	       "pxor     %%mm7,%%mm7\n\t"
	       "pcmpgtw  %%mm1,%%mm7\n\t" // mm7: <0 masks (0xFFFF iff in<0)

	       "movq     %%mm1,%%mm0\n\t"
	       "psllw    $1,%%mm0\n\t"
	       "pand     %%mm7,%%mm0\n\t"
	       "psubw    %%mm0,%%mm1\n\t" // mm1=abs(in)

	       // ---- quantize

	       "paddw    (%3),%%mm1\n\t"
	       "pmulhw   (%2),%%mm1\n\t"


	       // ---- restore sign

	       "movq     %%mm1,%%mm2\n\t"
	       "psllw    $1,%%mm2\n\t"
	       "pand     %%mm7,%%mm2\n\t"
	       "psubw    %%mm2,%%mm1\n\t"

	       "movq     %%mm1,(%1)\n\t"

	       : : "r"(inp),"r"(outp),"r"(qtab), "r"(otab)
	       );

	    inp += 4;
	    outp += 4;
	    qtab += 4;
	    otab += 4;
	  }
      }

    __asm__
      (
       "emms\n\t"
       );


#if 0
    for (int i=0;i<64;i++)
      {
	int v0 = copy[0][i]*16 / (qmatrix.mat[0][i]*qscale);
	//out.coeff[0][i] = v0;
	if (v0 != out.coeff[0][i])
	  cout << "ERROR: " << i << ": " << in.coeff[0][i] << " " << out.coeff[0][i] << " | " << v0 << endl;
      }
#endif
  }


  void MP2_Quantizer_MMX::Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  {
    const uint16* qtab = &qmat_inter[qscale][0][0];
    const int16* inp  = &in.coeff[0][0];
    int16* outp = &out.coeff[0][0];

#if 0
    int16 copy[8][8];

    for (int i=0;i<64;i++)
      copy[0][i] = in.coeff[0][i];
#endif


    if (d_high_precision)
      {
	for (int i=0;i<64/4;i++)
	  {
	    __asm__
	      (
	       // ---- calc abs(in)

	       "movq     (%0),%%mm1\n\t"  // signed input values
	       "pxor     %%mm7,%%mm7\n\t"
	       "pcmpgtw  %%mm1,%%mm7\n\t" // mm7: <0 masks (0xFFFF iff in<0)

	       "movq     %%mm1,%%mm0\n\t"
	       "psllw    $1,%%mm0\n\t"
	       "pand     %%mm7,%%mm0\n\t"
	       "psubw    %%mm0,%%mm1\n\t" // mm1=abs(in)

	       // ---- quantize

#if 1
	       "movq     %%mm1,%%mm2\n\t"
	       "pmullw   (%2),%%mm2\n\t"

	       "movq     (%2),%%mm3\n\t"
	       "psrlw    $3,%%mm3\n\t"
	       "psrlw    $1,%%mm2\n\t"
	       "paddw    %%mm3,%%mm2\n\t"
	       "psrlw    $15,%%mm2\n\t"
#endif

	       "pmulhw   (%2),%%mm1\n\t"
	       "paddw    %%mm2,%%mm1\n\t"


	       // ---- restore sign

	       "movq     %%mm1,%%mm2\n\t"
	       "psllw    $1,%%mm2\n\t"
	       "pand     %%mm7,%%mm2\n\t"
	       "psubw    %%mm2,%%mm1\n\t"

	       "movq     %%mm1,(%1)\n\t"

	       : : "r"(inp),"r"(outp),"r"(qtab)
	       );

	    inp += 4;
	    outp += 4;
	    qtab += 4;
	  }
      }
    else
      {
	for (int i=0;i<64/4;i++)
	  {
	    __asm__
	      (
	       // ---- calc abs(in)

	       "movq     (%0),%%mm1\n\t"  // signed input values
	       "pxor     %%mm7,%%mm7\n\t"
	       "pcmpgtw  %%mm1,%%mm7\n\t" // mm7: <0 masks (0xFFFF iff in<0)

	       "movq     %%mm1,%%mm0\n\t"
	       "psllw    $1,%%mm0\n\t"
	       "pand     %%mm7,%%mm0\n\t"
	       "psubw    %%mm0,%%mm1\n\t" // mm1=abs(in)

	       // ---- quantize

	       "pmulhw   (%2),%%mm1\n\t"


	       // ---- restore sign

	       "movq     %%mm1,%%mm2\n\t"
	       "psllw    $1,%%mm2\n\t"
	       "pand     %%mm7,%%mm2\n\t"
	       "psubw    %%mm2,%%mm1\n\t"

	       "movq     %%mm1,(%1)\n\t"

	       : : "r"(inp),"r"(outp),"r"(qtab)
	       );

	    inp += 4;
	    outp += 4;
	    qtab += 4;
	  }
      }

    __asm__
      (
       "emms\n\t"
       );


#if 0
    for (int i=0;i<64;i++)
      {
	int v0 = copy[0][i]*16 / (qmatrix.mat[0][i]*qscale);
	//out.coeff[0][i] = v0;
	if (v0 != out.coeff[0][i])
	  cout << "ERROR: " << i << ": " << in.coeff[0][i] << " " << out.coeff[0][i] << " | " << v0 << endl;
      }
#endif
  }


  bool MP2_Quantizer_MMX::Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,
					 int qscale, int32& out_of_range)
  {
    Quantize_Intra(out,in,qscale);
    if (!out.IsACInRange(d_clip_low, d_clip_high)) out_of_range = 1; else out_of_range = 0;
    return !out.IsAllZero();
  }


  bool MP2_Quantizer_MMX::Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,
					 int qscale, int32& out_of_range)
  {
    Quantize_Inter(out,in,qscale);
    if (!out.IsInRange(d_clip_low, d_clip_high)) out_of_range = 1; else out_of_range = 0;
    return !out.IsAllZero();
  }



  void MP2_Quantizer_MMX::Dequantize_Inter(DCTBlock_8x8& outb,const DCTBlock_8x8& inb,int qscale)
  {
    // As long as there is no overflow in the multiplication step, this dequantization should be
    // exactly identical to the scalar code.

    static const uint64 addone = 0x0001000100010001LL;

    const uint8* qtab = &d_inter.mat[0][0];
    const int16* inp  = &inb.coeff[0][0];
    uint64* q4 = &qscale_4times[qscale];

    int16* outp = &outb.coeff[0][0];


    for (int i=0;i<64/4;i++)
      {
	__asm__
	  (
	   // ---- calc abs(in)

	   "movq     (%0),%%mm1\n\t"  // signed input values
	   "pxor     %%mm7,%%mm7\n\t"
	   "pcmpgtw  %%mm1,%%mm7\n\t" // mm7: <0 masks (0xFFFF iff in<0)

	   "movq     %%mm1,%%mm0\n\t"
	   "psllw    $1,%%mm0\n\t"
	   "pand     %%mm7,%%mm0\n\t"
	   "psubw    %%mm0,%%mm1\n\t" // mm1=abs(in)

	   // ---- dequantize

	   "psllw    $1, %%mm1\n\t"    // in *= 2

	   // in += 1  if in!=0
	   "pxor     %%mm6,%%mm6\n\t"
	   "pcmpeqw  %%mm1,%%mm6\n\t"

	   "movq     %4, %%mm4\n\t"
	   "pandn    %%mm4,%%mm6\n\t"
	   "paddw    %%mm6,%%mm1\n\t"

	   "movd      (%2),%%mm3\n\t"  // qtab
	   "pxor      %%mm6,%%mm6\n\t"
	   "punpcklbw %%mm6,%%mm3\n\t"

	   "pmullw    %%mm3,%%mm1\n\t" // in *= QMAT
	   "pmullw    %3, %%mm1\n\t"   // in *= Q

	 
	   "psrlw    $5,%%mm1\n\t"     // in /= 32


	   // ---- restore sign

	   "movq     %%mm1,%%mm2\n\t"
	   "psllw    $1,%%mm2\n\t"
	   "pand     %%mm7,%%mm2\n\t"
	   "psubw    %%mm2,%%mm1\n\t"

	   "movq     %%mm1,(%1)\n\t"

	   : : "r"(inp),"r"(outp) ,"r"(qtab), "m"(*q4), "m"(addone)
	   );

	inp += 4;
	outp += 4;
	qtab += 4;
      }

    __asm__
      (
       "emms\n\t"
       );
  }


  void MP2_Quantizer_MMX::Dequantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  {
    // As long as there is no overflow in the multiplication step, this dequantization should be
    // exactly identical to the scalar code.

    const uint8* qtab = &d_intra.mat[0][0];
    const int16* inp  = &in.coeff[0][0];
    int16* outp = &out.coeff[0][0];


    const uint64 q4 = qscale_4times[qscale];

    int16 dc = in.coeff[0][0];

    for (int i=0;i<64/4;i++)
      {
	__asm__
	  (
	   // ---- calc abs(in)

	   "movq     (%0),%%mm1\n\t"  // signed input values
	   "pxor     %%mm7,%%mm7\n\t"
	   "pcmpgtw  %%mm1,%%mm7\n\t" // mm7: <0 masks (0xFFFF iff in<0)

	   "movq     %%mm1,%%mm0\n\t"
	   "psllw    $1,%%mm0\n\t"
	   "pand     %%mm7,%%mm0\n\t"
	   "psubw    %%mm0,%%mm1\n\t" // mm1=abs(in)

	   // ---- dequantize

	   "movd      (%2),%%mm3\n\t"  // qtab
	   "pxor      %%mm6,%%mm6\n\t"
	   "punpcklbw %%mm6,%%mm3\n\t"

	   "pmullw    %%mm3,%%mm1\n\t" // in *= QMAT
	   "pmullw    %3, %%mm1\n\t"   // in *= Q

	 
	   "psrlw    $4,%%mm1\n\t"     // in /= 16


	   // ---- restore sign

	   "movq     %%mm1,%%mm2\n\t"
	   "psllw    $1,%%mm2\n\t"
	   "pand     %%mm7,%%mm2\n\t"
	   "psubw    %%mm2,%%mm1\n\t"

	   "movq     %%mm1,(%1)\n\t"

	   : : "r"(inp),"r"(outp) ,"r"(qtab), "m"(q4)
	   );

	inp += 4;
	outp += 4;
	qtab += 4;
      }

    __asm__
      (
       "emms\n\t"
       );

    out.coeff[0][0] = dc << d_dc_shift;
  }

}
