/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include <libvideocoding/mpeg2/encoder/quantize_scalar.hh>
#include <libvideocoding/mpeg2/syntax/video.hh>

#include <iostream>

namespace videocoding {
  using namespace std;


  void MP2_Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,
			  const QMatrix_8x8& qmatrix,int qscale,int dc_precision)
  {
    for (int i=0;i<8;i++)
      for (int j=0;j<8;j++)
	if (i==0 && j==0)
	  {
	    dc_precision -= 8;
	    int dcoffset = 4>>dc_precision;

	    if (in.coeff[i][j]>0)
	      out.coeff[i][j] = (in.coeff[i][j]+dcoffset) >> (3-dc_precision);
	    else
	      out.coeff[i][j] = (in.coeff[i][j]-dcoffset) >> (3-dc_precision);
	  }
	else
	  {
	    out.coeff[i][j] = (32*in.coeff[i][j]+sign(in.coeff[i][j])*qmatrix.mat[i][j]*qscale) /
	      (qmatrix.mat[i][j]*qscale*2);
	  }
  }


  void MP2_Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,
			  const QMatrix_8x8& qmatrix,int qscale)
  {
    for (int i=0;i<8;i++)
      for (int j=0;j<8;j++)
	{
	  out.coeff[i][j] = in.coeff[i][j]*16 / (qmatrix.mat[i][j]*qscale);
	}
  }


  void MP2_Dequantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,
			    const QMatrix_8x8& qmatrix,int qscale,int dc_precision)
  {
    for (int i=0;i<8;i++)
      for (int j=0;j<8;j++)
	if (i==0 && j==0)
	  {
	    dc_precision -= 8;
	    out.coeff[i][j] = in.coeff[i][j] << (3-dc_precision);
	  }
	else
	  {
	    out.coeff[i][j] = in.coeff[i][j]*qscale*qmatrix.mat[i][j]/16;
	  }
  }


  void MP2_Dequantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,
			    const QMatrix_8x8& qmatrix,int qscale)
  {
    for (int i=0;i<8;i++)
      for (int j=0;j<8;j++)
	{
	  if (in.coeff[i][j]==0)
	    { }
	  else if (in.coeff[i][j]>0)
	    out.coeff[i][j] = (in.coeff[i][j]*2+1) * qmatrix.mat[i][j] * qscale / 32;
	  else
	    out.coeff[i][j] = (in.coeff[i][j]*2-1) * qmatrix.mat[i][j] * qscale / 32;
	}
  }


  void MP2_Quantizer_Scalar::Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  { MP2_Quantize_Intra(out,in,d_intra,qscale,d_dc_precision); }

  void MP2_Quantizer_Scalar::Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  { MP2_Quantize_Inter(out,in,d_inter,qscale); }
  
  void MP2_Quantizer_Scalar::Dequantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  { MP2_Dequantize_Intra(out,in,d_intra,qscale,d_dc_precision); }

  void MP2_Quantizer_Scalar::Dequantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,int qscale)
  { MP2_Dequantize_Inter(out,in,d_inter,qscale); }

  bool MP2_Quantizer_Scalar::Quantize_Intra(DCTBlock_8x8& out,const DCTBlock_8x8& in,
					    int qscale, int32& out_of_range)
  {
    Quantize_Intra(out,in,qscale);
    if (!out.IsACInRange(d_clip_low, d_clip_high)) out_of_range = 1; else out_of_range = 0;
    return !out.IsAllZero();
  }


  bool MP2_Quantizer_Scalar::Quantize_Inter(DCTBlock_8x8& out,const DCTBlock_8x8& in,
					    int qscale, int32& out_of_range)
  {
    Quantize_Inter(out,in,qscale);
    if (!out.IsInRange(d_clip_low, d_clip_high)) out_of_range = 1; else out_of_range = 0;
    return !out.IsAllZero();
  }


  MP2_Quantizer_Scalar::MP2_Quantizer_Scalar()
  {
    SetDCPrecision(8);
    SetIntraQMatrix(MP2_DefaultQuantMatrix_Intra);
    SetInterQMatrix(MP2_DefaultQuantMatrix_Inter);
  }

}
