/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/mpeg2/syntax/dct.hh"

namespace videocoding {

  static int getlog2(int n)
  {
    static int logtab[] = { 0,1,2,4,8,16,32,64,128,256,512,1024,2048 };

    assert(n<=2048);
    int i=0;
    while (logtab[i] <= n)
      i++;

    return i-1;
  }


  struct DirectVLCTableEntry
  {
    uint16 bits; uint16 nBits;
  };

  static const DirectVLCTableEntry vlc_DClum_tab[12]=
    {
      /* Tabelle geklaut aus MSSG-Encoder */
      {0x0004,3}, {0x0000,2}, {0x0001,2}, {0x0005,3}, {0x0006,3}, {0x000e,4},
      {0x001e,5}, {0x003e,6}, {0x007e,7}, {0x00fe,8}, {0x01fe,9}, {0x01ff,9}
    };

  static const DirectVLCTableEntry vlc_DCchrom_tab[12]=
    {
      /* Tabelle geklaut aus MSSG-Encoder */
      {0x0000,2}, {0x0001,2}, {0x0002,2}, {0x0006,3}, {0x000e,4}, {0x001e,5},
      {0x003e,6}, {0x007e,7}, {0x00fe,8}, {0x01fe,9}, {0x03fe,10},{0x03ff,10}
    };


  static const struct DirectVLCTableEntry dct_code_B14a[2][40]=
    {
      /* run = 0, level = 1...40 */
      {
	{0x03, 2}, {0x04, 4}, {0x05, 5}, {0x06, 7},
	{0x26, 8}, {0x21, 8}, {0x0a,10}, {0x1d,12},
	{0x18,12}, {0x13,12}, {0x10,12}, {0x1a,13},
	{0x19,13}, {0x18,13}, {0x17,13}, {0x1f,14},
	{0x1e,14}, {0x1d,14}, {0x1c,14}, {0x1b,14},
	{0x1a,14}, {0x19,14}, {0x18,14}, {0x17,14},
	{0x16,14}, {0x15,14}, {0x14,14}, {0x13,14},
	{0x12,14}, {0x11,14}, {0x10,14}, {0x18,15},
	{0x17,15}, {0x16,15}, {0x15,15}, {0x14,15},
	{0x13,15}, {0x12,15}, {0x11,15}, {0x10,15}
      },
      /* run = 1, level = 1...18 */
      {
	{0x03, 3}, {0x06, 6}, {0x25, 8}, {0x0c,10},
	{0x1b,12}, {0x16,13}, {0x15,13}, {0x1f,15},
	{0x1e,15}, {0x1d,15}, {0x1c,15}, {0x1b,15},
	{0x1a,15}, {0x19,15}, {0x13,16}, {0x12,16},
	{0x11,16}, {0x10,16}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}
      }
    };


  static const struct DirectVLCTableEntry dct_code_B14b[30][5]=
    {
      /* run = 2...31, level = 1...5 */
      {{0x05, 4}, {0x04, 7}, {0x0b,10}, {0x14,12}, {0x14,13}},
      {{0x07, 5}, {0x24, 8}, {0x1c,12}, {0x13,13}, {0x00, 0}},
      {{0x06, 5}, {0x0f,10}, {0x12,12}, {0x00, 0}, {0x00, 0}},
      {{0x07, 6}, {0x09,10}, {0x12,13}, {0x00, 0}, {0x00, 0}},
      {{0x05, 6}, {0x1e,12}, {0x14,16}, {0x00, 0}, {0x00, 0}},
      {{0x04, 6}, {0x15,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x07, 7}, {0x11,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x05, 7}, {0x11,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x27, 8}, {0x10,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x23, 8}, {0x1a,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x22, 8}, {0x19,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x20, 8}, {0x18,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x0e,10}, {0x17,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x0d,10}, {0x16,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x08,10}, {0x15,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1f,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1a,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x19,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x17,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x16,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1f,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1e,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1d,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1c,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1b,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1f,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1e,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1d,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1c,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1b,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}}
    };


  static const struct DirectVLCTableEntry dct_code_B15a[2][40]=
    {
      /* run = 0, level = 1...40 */
      {
	{0x02, 2}, {0x06, 3}, {0x07, 4}, {0x1c, 5},
	{0x1d, 5}, {0x05, 6}, {0x04, 6}, {0x7b, 7},
	{0x7c, 7}, {0x23, 8}, {0x22, 8}, {0xfa, 8},
	{0xfb, 8}, {0xfe, 8}, {0xff, 8}, {0x1f,14},
	{0x1e,14}, {0x1d,14}, {0x1c,14}, {0x1b,14},
	{0x1a,14}, {0x19,14}, {0x18,14}, {0x17,14},
	{0x16,14}, {0x15,14}, {0x14,14}, {0x13,14},
	{0x12,14}, {0x11,14}, {0x10,14}, {0x18,15},
	{0x17,15}, {0x16,15}, {0x15,15}, {0x14,15},
	{0x13,15}, {0x12,15}, {0x11,15}, {0x10,15}
      },
      /* run = 1, level = 1...18 */
      {
	{0x02, 3}, {0x06, 5}, {0x79, 7}, {0x27, 8},
	{0x20, 8}, {0x16,13}, {0x15,13}, {0x1f,15},
	{0x1e,15}, {0x1d,15}, {0x1c,15}, {0x1b,15},
	{0x1a,15}, {0x19,15}, {0x13,16}, {0x12,16},
	{0x11,16}, {0x10,16}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0},
	{0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}
      }
    };

  static const struct DirectVLCTableEntry dct_code_B15b[30][5]=
    {
      /* run = 2...31, level = 1...5 */
      {{0x05, 5}, {0x07, 7}, {0xfc, 8}, {0x0c,10}, {0x14,13}},
      {{0x07, 5}, {0x26, 8}, {0x1c,12}, {0x13,13}, {0x00, 0}},
      {{0x06, 6}, {0xfd, 8}, {0x12,12}, {0x00, 0}, {0x00, 0}},
      {{0x07, 6}, {0x04, 9}, {0x12,13}, {0x00, 0}, {0x00, 0}},
      {{0x06, 7}, {0x1e,12}, {0x14,16}, {0x00, 0}, {0x00, 0}},
      {{0x04, 7}, {0x15,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x05, 7}, {0x11,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x78, 7}, {0x11,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x7a, 7}, {0x10,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x21, 8}, {0x1a,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x25, 8}, {0x19,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x24, 8}, {0x18,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x05, 9}, {0x17,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x07, 9}, {0x16,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x0d,10}, {0x15,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1f,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1a,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x19,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x17,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x16,12}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1f,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1e,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1d,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1c,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1b,13}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1f,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1e,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1d,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1c,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}},
      {{0x1b,16}, {0x00, 0}, {0x00, 0}, {0x00, 0}, {0x00, 0}}
    };




  MP2_DCTBlockEncoder::MP2_DCTBlockEncoder()
    : d_bs(NULL)
  {
    SetParams(true,false,false);
  }


  void MP2_DCTBlockEncoder::SetParams(bool UseMPEG1,bool UseB15,bool UseAlternateScan)
  {
    if (d_mpeg1)
      {
	assert(UseB15 == false);
	assert(UseAlternateScan == false);
      }

    d_mpeg1 = UseMPEG1;
    d_b15   = UseB15;
    d_scan = AskScan(UseAlternateScan);
  }


  void MP2_DCTBlockEncoder::SetParams(const MP2_PictureHeader& pichdr)
  {
    assert(pichdr.seqhdr);

    SetParams(!pichdr.seqhdr->IsMPEG2,
	      pichdr.intra_vlc_format,
	      pichdr.alternate_scan);
  }


  /*
    TODO: this code can be optimized a lot.
  */
  void MP2_DCTBlockEncoder::Write(const DCTBlock_8x8& blk,int& DC_predictor,bool isY,bool intra)
  {
    int next_coeff;

    if (intra)
      {
	// code DC-value

	short dcdiff = blk.coeff[0][0]-DC_predictor;
	DC_predictor=blk.coeff[0][0];

	int size = getlog2(abs(dcdiff));
	int diffbits;
	if (dcdiff<0) diffbits = ~(-dcdiff);
	else          diffbits =    dcdiff;

	assert(size >= 0);
	assert(size <= 11);

	if (isY)
	  {
	    d_bs->WriteBits(vlc_DClum_tab[size].bits,
			    vlc_DClum_tab[size].nBits);
	  }
	else
	  {
	    d_bs->WriteBits(vlc_DCchrom_tab[size].bits,
			    vlc_DCchrom_tab[size].nBits);
	  }

	if (size>0)
	  d_bs->WriteBitsMasked(diffbits,size);

	next_coeff = 1; // continue with first AC-coefficient
      }
    else
      {
	next_coeff = 0; // inter block -> code all coefficients as run-value pairs
      }


    const Scan* s = &d_scan[next_coeff];

    int run=0;
    while (next_coeff<64)
      {
	short value = blk.coeff[ s->v ][ s->u ];

	if (value==0)
	  {
	    run++;    // increase length of zero-run
	  }
	else
	  {
	    assert(run>=0);
	    assert(run<=63);
	    assert(value>=-2047);
	    assert(value<= 2047);
	    assert(value!=0);

	    int absvalue = abs(value);

	    bool InTab = false;
	    bool TabA = false; // Initialization not necesarily needed, but calms down the compiler.

	    uint32 bits=0;     // This initialialization not needed, either.
	    int nBits = 0;

	    if (next_coeff==0 && !d_b15 && run==0 && absvalue==1)
	      {
		if (value>0) d_bs->WriteBits(2,2);
		else         d_bs->WriteBits(3,2);
	      }
	    else
	      {
		if (run<=1)
		  { if (absvalue<=40) { InTab = true; TabA=true;  } }
		else if (run<=31)
		  { if (absvalue<=5)  { InTab = true; TabA=false; } }

		if (InTab)
		  {
		    if (d_b15)
		      {
			if (TabA)
			  {
			    bits  = dct_code_B15a[run][absvalue-1].bits;
			    nBits = dct_code_B15a[run][absvalue-1].nBits;
			  }
			else
			  {
			    bits  = dct_code_B15b[run-2][absvalue-1].bits;
			    nBits = dct_code_B15b[run-2][absvalue-1].nBits;
			  }
		      }
		    else
		      {
			if (TabA)
			  {
			    bits  = dct_code_B14a[run][absvalue-1].bits;
			    nBits = dct_code_B14a[run][absvalue-1].nBits;
			  }
			else
			  {
			    bits  = dct_code_B14b[run-2][absvalue-1].bits;
			    nBits = dct_code_B14b[run-2][absvalue-1].nBits;
			  }
		      }
		  }

		if (nBits>0)
		  {
		    // append the sign bit

		    nBits++; bits <<= 1;
		    if (value<0) bits |= 1;
		    d_bs->WriteBits(bits,nBits);
		  }
		else
		  {
		    /* code run-value pair as escape sequence */

		    if (d_mpeg1)
		      {
			d_bs->WriteBits(1,6);
			d_bs->WriteBits(run,6);
			if (absvalue <= 127)
			  d_bs->WriteBitsMasked(value,8);
			else
			  {
			    if (value>0)
			      {
				assert(value>=128);
				assert(value<=255);
				d_bs->WriteBitsMasked(value,16);
			      }
			    else
			      {
				assert(value>=-255);
				assert(value<=-128);
				d_bs->WriteBits(0x80,8);
				d_bs->WriteBitsMasked(value,8);
			      }
			  }
		      }
		    else
		      {
			d_bs->WriteBits(1,6);
			d_bs->WriteBits(run,6);
			d_bs->WriteBitsMasked(value,12);
		      }
		  }
	      }

	    run=0;
	  }

	s++;
	next_coeff++;
      }

    // write EOB

    if (!d_b15) d_bs->WriteBits(2,2);
    else        d_bs->WriteBits(6,4);
  }



  int MP2_DCTBlockEncoder::QueryCodeLength(int run,int value,bool first_in_intra) const
  {
    int absvalue = abs(value);

    bool InTab = false;
    bool TabA = false; // Initialization not necesarily needed, but calms down the compiler.

    int nBits = 0;

    if (first_in_intra && !d_b15 && run==0 && absvalue==1)
      {
	return 2;
      }
    else
      {
	if (run<=1)
	  { if (absvalue<=40) { InTab = true; TabA=true;  } }
	else if (run<=31)
	  { if (absvalue<=5)  { InTab = true; TabA=false; } }

	if (InTab)
	  {
	    if (d_b15)
	      {
		if (TabA)
		  nBits = dct_code_B15a[run][absvalue-1].nBits;
		else
		  nBits = dct_code_B15b[run-2][absvalue-1].nBits;
	      }
	    else
	      {
		if (TabA)
		  nBits = dct_code_B14a[run][absvalue-1].nBits;
		else
		  nBits = dct_code_B14b[run-2][absvalue-1].nBits;
	      }
	  }

	if (nBits>0)
	  nBits++;
	else
	  {
	    /* escape sequence */

	    if (d_mpeg1)
	      {
		nBits += 12;
		if (absvalue <= 127)
		  nBits += 8;
		else
		  nBits += 16;
	      }
	    else
	      nBits += 24;
	  }
      }

    return nBits;
  }


  int MP2_DCTBlockEncoder::IsEscapeCode(int run,int value) const
  {
    int absvalue = abs(value);

    bool InTab = false;
    bool TabA = false; // Initialization not necesarily needed, but calms down the compiler.

    int nBits=0;

    if (run<=1)
      { if (absvalue<=40) { InTab = true; TabA=true;  } }
    else if (run<=31)
      { if (absvalue<=5)  { InTab = true; TabA=false; } }

    if (InTab)
      {
	if (d_b15)
	  {
	    if (TabA)
	      nBits = dct_code_B15a[run][absvalue-1].nBits;
	    else
	      nBits = dct_code_B15b[run-2][absvalue-1].nBits;
	  }
	else
	  {
	    if (TabA)
	      nBits = dct_code_B14a[run][absvalue-1].nBits;
	    else
	      nBits = dct_code_B14b[run-2][absvalue-1].nBits;
	  }
      }

    if (nBits>0)
      return false;
    else
      return true;
  }



  // -------------------------------- decoder ----------------------------------



  static const int scantab[2][64] =
    {
      { /* Zig-Zag scan pattern  */
	0,1,8,16,9,2,3,10,17,24,32,25,18,11,4,5,
	12,19,26,33,40,48,41,34,27,20,13,6,7,14,21,28,
	35,42,49,56,57,50,43,36,29,22,15,23,30,37,44,51,
	58,59,52,45,38,31,39,46,53,60,61,54,47,55,62,63
      },
      { /* Alternate scan pattern */
	0,8,16,24,1,9,2,10,17,25,32,40,48,56,57,49,
	41,33,26,18,3,11,4,12,19,27,34,42,50,58,35,43,
	51,59,20,28,5,13,6,14,21,29,36,44,52,60,37,45,
	53,61,22,30,7,15,23,31,38,46,54,62,39,47,55,63
      }
    };


  // -------------------------------------------------------------------------------------------
  //                                           DCTBLK
  // -------------------------------------------------------------------------------------------

  typedef struct {
    uint8 run;
    uint8 level;
    uint8 len;
  } DCTtabb;



  static DCTtabb DCT_16 [] = {
    {129, 0, 0}, {129, 0, 0}, {129, 0, 0}, {129, 0, 0},
    {129, 0, 0}, {129, 0, 0}, {129, 0, 0}, {129, 0, 0},
    {129, 0, 0}, {129, 0, 0}, {129, 0, 0}, {129, 0, 0},
    {129, 0, 0}, {129, 0, 0}, {129, 0, 0}, {129, 0, 0},
    {  2,18, 0}, {  2,17, 0}, {  2,16, 0}, {  2,15, 0},
    {  7, 3, 0}, { 17, 2, 0}, { 16, 2, 0}, { 15, 2, 0},
    { 14, 2, 0}, { 13, 2, 0}, { 12, 2, 0}, { 32, 1, 0},
    { 31, 1, 0}, { 30, 1, 0}, { 29, 1, 0}, { 28, 1, 0}
  };

  static DCTtabb DCT_15 [] = {
    {  1,40,15}, {  1,39,15}, {  1,38,15}, {  1,37,15},
    {  1,36,15}, {  1,35,15}, {  1,34,15}, {  1,33,15},
    {  1,32,15}, {  2,14,15}, {  2,13,15}, {  2,12,15},
    {  2,11,15}, {  2,10,15}, {  2, 9,15}, {  2, 8,15},
    {  1,31,14}, {  1,31,14}, {  1,30,14}, {  1,30,14},
    {  1,29,14}, {  1,29,14}, {  1,28,14}, {  1,28,14},
    {  1,27,14}, {  1,27,14}, {  1,26,14}, {  1,26,14},
    {  1,25,14}, {  1,25,14}, {  1,24,14}, {  1,24,14},
    {  1,23,14}, {  1,23,14}, {  1,22,14}, {  1,22,14},
    {  1,21,14}, {  1,21,14}, {  1,20,14}, {  1,20,14},
    {  1,19,14}, {  1,19,14}, {  1,18,14}, {  1,18,14},
    {  1,17,14}, {  1,17,14}, {  1,16,14}, {  1,16,14}
  };

  static DCTtabb DCT_13 [] = {
    { 11, 2,13}, { 10, 2,13}, {  6, 3,13}, {  4, 4,13},
    {  3, 5,13}, {  2, 7,13}, {  2, 6,13}, {  1,15,13},
    {  1,14,13}, {  1,13,13}, {  1,12,13}, { 27, 1,13},
    { 26, 1,13}, { 25, 1,13}, { 24, 1,13}, { 23, 1,13},
    {  1,11,12}, {  1,11,12}, {  9, 2,12}, {  9, 2,12},
    {  5, 3,12}, {  5, 3,12}, {  1,10,12}, {  1,10,12},
    {  3, 4,12}, {  3, 4,12}, {  8, 2,12}, {  8, 2,12},
    { 22, 1,12}, { 22, 1,12}, { 21, 1,12}, { 21, 1,12},
    {  1, 9,12}, {  1, 9,12}, { 20, 1,12}, { 20, 1,12},
    { 19, 1,12}, { 19, 1,12}, {  2, 5,12}, {  2, 5,12},
    {  4, 3,12}, {  4, 3,12}, {  1, 8,12}, {  1, 8,12},
    {  7, 2,12}, {  7, 2,12}, { 18, 1,12}, { 18, 1,12}
  };

  static DCTtabb DCT_B14_10 [] = {
    { 17, 1,10}, {  6, 2,10}, {  1, 7,10}, {  3, 3,10},
    {  2, 4,10}, { 16, 1,10}, { 15, 1,10}, {  5, 2,10}
  };

  static DCTtabb DCT_B14_8 [] = {
    { 65, 0, 6}, { 65, 0, 6}, { 65, 0, 6}, { 65, 0, 6},
    {  3, 2, 7}, {  3, 2, 7}, { 10, 1, 7}, { 10, 1, 7},
    {  1, 4, 7}, {  1, 4, 7}, {  9, 1, 7}, {  9, 1, 7},
    {  8, 1, 6}, {  8, 1, 6}, {  8, 1, 6}, {  8, 1, 6},
    {  7, 1, 6}, {  7, 1, 6}, {  7, 1, 6}, {  7, 1, 6},
    {  2, 2, 6}, {  2, 2, 6}, {  2, 2, 6}, {  2, 2, 6},
    {  6, 1, 6}, {  6, 1, 6}, {  6, 1, 6}, {  6, 1, 6},
    { 14, 1, 8}, {  1, 6, 8}, { 13, 1, 8}, { 12, 1, 8},
    {  4, 2, 8}, {  2, 3, 8}, {  1, 5, 8}, { 11, 1, 8}
  };

  static DCTtabb DCT_B14AC_5 [] = {
    {  1, 3, 5}, {  5, 1, 5}, {  4, 1, 5},
    {  1, 2, 4}, {  1, 2, 4}, {  3, 1, 4}, {  3, 1, 4},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {129, 0, 2}, {129, 0, 2}, {129, 0, 2}, {129, 0, 2},
    {129, 0, 2}, {129, 0, 2}, {129, 0, 2}, {129, 0, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}
  };

  static DCTtabb DCT_B14DC_5 [] = {
    {  1, 3, 5}, {  5, 1, 5}, {  4, 1, 5},
    {  1, 2, 4}, {  1, 2, 4}, {  3, 1, 4}, {  3, 1, 4},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1},
    {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1},
    {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1},
    {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1}, {  1, 1, 1}
  };

  static DCTtabb DCT_B15_10 [] = {
    {  6, 2, 9}, {  6, 2, 9}, { 15, 1, 9}, { 15, 1, 9},
    {  3, 4,10}, { 17, 1,10}, { 16, 1, 9}, { 16, 1, 9}
  };

  static DCTtabb DCT_B15_8 [] = {
    { 65, 0, 6}, { 65, 0, 6}, { 65, 0, 6}, { 65, 0, 6},
    {  8, 1, 7}, {  8, 1, 7}, {  9, 1, 7}, {  9, 1, 7},
    {  7, 1, 7}, {  7, 1, 7}, {  3, 2, 7}, {  3, 2, 7},
    {  1, 7, 6}, {  1, 7, 6}, {  1, 7, 6}, {  1, 7, 6},
    {  1, 6, 6}, {  1, 6, 6}, {  1, 6, 6}, {  1, 6, 6},
    {  5, 1, 6}, {  5, 1, 6}, {  5, 1, 6}, {  5, 1, 6},
    {  6, 1, 6}, {  6, 1, 6}, {  6, 1, 6}, {  6, 1, 6},
    {  2, 5, 8}, { 12, 1, 8}, {  1,11, 8}, {  1,10, 8},
    { 14, 1, 8}, { 13, 1, 8}, {  4, 2, 8}, {  2, 4, 8},
    {  3, 1, 5}, {  3, 1, 5}, {  3, 1, 5}, {  3, 1, 5},
    {  3, 1, 5}, {  3, 1, 5}, {  3, 1, 5}, {  3, 1, 5},
    {  2, 2, 5}, {  2, 2, 5}, {  2, 2, 5}, {  2, 2, 5},
    {  2, 2, 5}, {  2, 2, 5}, {  2, 2, 5}, {  2, 2, 5},
    {  4, 1, 5}, {  4, 1, 5}, {  4, 1, 5}, {  4, 1, 5},
    {  4, 1, 5}, {  4, 1, 5}, {  4, 1, 5}, {  4, 1, 5},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3}, {  2, 1, 3},
    {129, 0, 4}, {129, 0, 4}, {129, 0, 4}, {129, 0, 4},
    {129, 0, 4}, {129, 0, 4}, {129, 0, 4}, {129, 0, 4},
    {129, 0, 4}, {129, 0, 4}, {129, 0, 4}, {129, 0, 4},
    {129, 0, 4}, {129, 0, 4}, {129, 0, 4}, {129, 0, 4},
    {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4},
    {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4},
    {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4},
    {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4}, {  1, 3, 4},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2}, {  1, 1, 2},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3}, {  1, 2, 3},
    {  1, 4, 5}, {  1, 4, 5}, {  1, 4, 5}, {  1, 4, 5},
    {  1, 4, 5}, {  1, 4, 5}, {  1, 4, 5}, {  1, 4, 5},
    {  1, 5, 5}, {  1, 5, 5}, {  1, 5, 5}, {  1, 5, 5},
    {  1, 5, 5}, {  1, 5, 5}, {  1, 5, 5}, {  1, 5, 5},
    { 10, 1, 7}, { 10, 1, 7}, {  2, 3, 7}, {  2, 3, 7},
    { 11, 1, 7}, { 11, 1, 7}, {  1, 8, 7}, {  1, 8, 7},
    {  1, 9, 7}, {  1, 9, 7}, {  1,12, 8}, {  1,13, 8},
    {  3, 3, 8}, {  5, 2, 8}, {  1,14, 8}, {  1,15, 8}
  };




  inline int DequantizeIntra(int value,int qscale,int matrix)
  {
    return value*qscale*matrix/16;
  }

  inline int DequantizeInter(int value,int qscale,int matrix)
  {
    if (value>0)
      return (2*value+1) * matrix * qscale / 32;
    else
      return (2*value-1) * matrix * qscale / 32;
  }

  inline void Saturate(int& value)
  {
    if (value<-2048) value=-2048;
    else if (value> 2047) value= 2047;
  }



  typedef struct {
    uint8 size;
    uint8 len;
  } DCtab;

  static DCtab DC_lum_5 [] = {  /* TODO: why is this table not used? */
    {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
    {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
    {0, 3}, {0, 3}, {0, 3}, {0, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3},
    {4, 3}, {4, 3}, {4, 3}, {4, 3}, {5, 4}, {5, 4}, {6, 5}
  };

  static DCtab DC_long [] = {
    {6, 5}, {6, 5}, {6, 5}, {6, 5}, {6, 5}, {6, 5}, { 6, 5}, { 6, 5},
    {6, 5}, {6, 5}, {6, 5}, {6, 5}, {6, 5}, {6, 5}, { 6, 5}, { 6, 5},
    {7, 6}, {7, 6}, {7, 6}, {7, 6}, {7, 6}, {7, 6}, { 7, 6}, { 7, 6},
    {8, 7}, {8, 7}, {8, 7}, {8, 7}, {9, 8}, {9, 8}, {10, 9}, {11, 9}
  };


  static DCtab DC_lum_7 [] = {
    {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
    {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
    {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
    {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},

    {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
    {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
    {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
    {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},

    {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3},
    {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3}, {0, 3},

    {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3},
    {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3},

    {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3},
    {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3}, {4, 3},
    {5, 4}, {5, 4}, {5, 4}, {5, 4}, {5, 4}, {5, 4}, {5, 4}, {5, 4},
    {6, 5}, {6, 5}, {6, 5}, {6, 5}, {7, 6}, {7, 6}, {8, 7}
  };



  static inline int get_luma_dc_dct_diff (BitReader_Fast& bs)
  {
    bs.Fill16Bits();

#define bit_buf bs.d_buffer
#define bits bs.d_freebits
    DCtab * tab;
    int size;
    int dc_diff;

    if (bit_buf < 0xfe000000) {
      /* Max. codelength = 7 + max. size = 8  <= 15 , no refill needed. */

      tab = DC_lum_7 + UBITS (bit_buf, 7);
      size = tab->size;

      if (size) {
	bits += tab->len + size;
	bit_buf <<= tab->len;
	dc_diff = UBITS (bit_buf, size) - UBITS (SBITS (~bit_buf, 1), size);
	bit_buf <<= size;
	return dc_diff;
      } else {
	DUMPBITS (bit_buf, bits, 3);
	return 0;
      }
    } else {
      tab = DC_long - 0x1e0 + UBITS (bit_buf, 9);
      size = tab->size;
      DUMPBITS(bit_buf,bits,tab->len);
      bs.Fill16Bits();
      dc_diff = UBITS (bit_buf, size) - UBITS (SBITS (~bit_buf, 1), size);
      DUMPBITS(bit_buf,bits,size);
      return dc_diff;
    }
#undef bit_buf
#undef bits
  }



  // -------------- chroma

  static DCtab DC_chrom_5 [] = {
    {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2},
    {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
    {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
    {3, 3}, {3, 3}, {3, 3}, {3, 3}, {4, 4}, {4, 4}, {5, 5}
  };


  static inline int get_chroma_dc_dct_diff (BitReader_Fast& bs)
  {
    bs.Fill16Bits();

#define bit_buf bs.d_buffer
#define bits bs.d_freebits
    DCtab * tab;
    int size;
    int dc_diff;

    if (bit_buf < 0xf8000000) {
      /* Max. codelength = 5 + max. size = 5  <= 16 , no refill needed. */

      tab = DC_chrom_5 + UBITS (bit_buf, 5);
      size = tab->size;
      if (size) {
	bits += tab->len + size;
	bit_buf <<= tab->len;
	dc_diff = UBITS (bit_buf, size) - UBITS (SBITS (~bit_buf, 1), size);
	bit_buf <<= size;
	return dc_diff;
      } else {
	DUMPBITS (bit_buf, bits, 2);
	return 0;
      }
    } else {
      tab = DC_long - 0x3e0 + UBITS (bit_buf, 10);
      size = tab->size;
      int len = tab->len+1;
      if (size>=8)
	{
	  DUMPBITS(bit_buf,bits,len);
	  bs.Fill16Bits();
	  dc_diff = UBITS (bit_buf, size) - UBITS (SBITS (~bit_buf, 1), size);
	  DUMPBITS(bit_buf,bits,size);
	}
      else
	{
	  bits += len+size;
	  bit_buf <<= len;
	  dc_diff = UBITS (bit_buf, size) - UBITS (SBITS (~bit_buf, 1), size);
	  bit_buf <<= size;
	}
      return dc_diff;
    }
#undef bit_buf
#undef bits
  }



  static void get_mpeg1_intra_block(class BitReader_Fast& bs,short* coeff,const int* scan,
				    int quantizer_scale,
				    const uint8* quant_matrix)
  {
    int i;
    int j;
    int val;
    DCTtabb * tab;
    uint32 bit_buf;
    int bits;

    i = 0;
    bs.MakeLocalCopy(bit_buf,bits);

    NEEDBITS (bs,bit_buf, bits);

    while (1) {
      if (bit_buf >= 0x28000000) {

	tab = DCT_B14AC_5 - 5 + UBITS (bit_buf, 5);

	i += tab->run;
	if (i >= 64)
	  break;  // end of block

      normal_code:
	j = scan[i];
	bit_buf <<= tab->len;
	bits += tab->len + 1;
	if (quant_matrix) val = DequantizeIntra(tab->level,quantizer_scale,quant_matrix[j]);
	else val = tab->level;

	// if (bitstream_get (1)) val = -val;
	val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);

	Saturate(val);
	coeff[j] = val;

	bit_buf <<= 1;
	NEEDBITS (bs,bit_buf, bits);

	continue;

      } else if (bit_buf >= 0x04000000) {

	tab = DCT_B14_8 - 4 + UBITS (bit_buf, 8);

	i += tab->run;
	if (i < 64)
	  goto normal_code;

	// escape code

	i += UBITS (bit_buf << 6, 6) - 64;
	if (i >= 64)
	  break;  // illegal, but check needed to avoid buffer overflow

	j = scan[i];

	DUMPBITS (bit_buf, bits, 12);
	NEEDBITS (bs,bit_buf, bits);
	val = SBITS (bit_buf, 8);
	if (! (val & 0x7f)) {
	  DUMPBITS (bit_buf, bits, 8);
	  val = UBITS (bit_buf, 8) + 2 * val;
	}
	if (quant_matrix) val = DequantizeIntra(val,quantizer_scale,quant_matrix[j]);

	Saturate(val);
	coeff[j] = val;

	DUMPBITS (bit_buf, bits, 8);
	NEEDBITS (bs,bit_buf, bits);

	continue;

      } else if (bit_buf >= 0x02000000) {
	tab = DCT_B14_10 - 8 + UBITS (bit_buf, 10);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00800000) {
	tab = DCT_13 - 16 + UBITS (bit_buf, 13);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00200000) {
	tab = DCT_15 - 16 + UBITS (bit_buf, 15);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else {
	tab = DCT_16 + UBITS (bit_buf, 16);
	bit_buf <<= 16;
	bit_buf |= bs.GetNextWord() << (bits + 16);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      }
      break;  // illegal, but check needed to avoid buffer overflow
    }
    DUMPBITS (bit_buf, bits, 2);        // dump end of block code

    bs.RestoreFromLocal(bit_buf,bits);
  }




  static void get_mpeg1_non_intra_block(class BitReader_Fast& bs,short* coeff,const int* scan,
					int quantizer_scale,
					const uint8* quant_matrix)
  {
    int i;
    int j;
    int val;
    DCTtabb * tab;
    uint32 bit_buf;
    int bits;

    i = -1;

    bs.MakeLocalCopy(bit_buf,bits);

    NEEDBITS (bs,bit_buf, bits);
    if (bit_buf >= 0x28000000) {
      tab = DCT_B14DC_5 - 5 + UBITS (bit_buf, 5);
      goto entry_1;
    } else
      goto entry_2;

    while (1) {
      if (bit_buf >= 0x28000000) {

	tab = DCT_B14AC_5 - 5 + UBITS (bit_buf, 5);

      entry_1:
	i += tab->run;
	if (i >= 64)
	  break;  // end of block

      normal_code:
	j = scan[i];
	bit_buf <<= tab->len;
	bits += tab->len + 1;
	if (quant_matrix) val = DequantizeInter(tab->level,quantizer_scale,quant_matrix[j]);
	else val = tab->level;

	// if (bitstream_get (1)) val = -val;
	val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);

	Saturate (val);
	coeff[j] = val;

	bit_buf <<= 1;
	NEEDBITS (bs,bit_buf, bits);

	continue;

      }

    entry_2:
      if (bit_buf >= 0x04000000) {

	tab = DCT_B14_8 - 4 + UBITS (bit_buf, 8);

	i += tab->run;
	if (i < 64)
	  goto normal_code;

	// escape code

	i += UBITS (bit_buf << 6, 6) - 64;
	if (i >= 64)
	  break;  // illegal, but check needed to avoid buffer overflow

	j = scan[i];

	DUMPBITS (bit_buf, bits, 12);
	NEEDBITS (bs,bit_buf, bits);
	val = SBITS (bit_buf, 8);
	if (! (val & 0x7f)) {
	  DUMPBITS (bit_buf, bits, 8);
	  val = UBITS (bit_buf, 8) + 2 * val;
	}
	if (quant_matrix)
	  {
	    val = DequantizeInter(val + SBITS (val, 1),quantizer_scale,quant_matrix[j]);
	  }
	else { val = val+ SBITS(val,1); }

	Saturate (val);
	coeff[j] = val;

	DUMPBITS (bit_buf, bits, 8);
	NEEDBITS (bs,bit_buf, bits);

	continue;

      } else if (bit_buf >= 0x02000000) {
	tab = DCT_B14_10 - 8 + UBITS (bit_buf, 10);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00800000) {
	tab = DCT_13 - 16 + UBITS (bit_buf, 13);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00200000) {
	tab = DCT_15 - 16 + UBITS (bit_buf, 15);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else {
	tab = DCT_16 + UBITS (bit_buf, 16);
	bit_buf <<= 16;
	bit_buf |= bs.GetNextWord() << (bits + 16);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      }
      break;  // illegal, but check needed to avoid buffer overflow
    }
    DUMPBITS (bit_buf, bits, 2);        // dump end of block code
    bs.RestoreFromLocal(bit_buf,bits);
  }


  static void get_intra_block_B14(class BitReader_Fast& bs,short* coeff,const int* scan,
				  int quantizer_scale,
				  const uint8* quant_matrix)
  {
    int i;
    int j;
    int val;
    int mismatch;
    DCTtabb * tab;
    uint32 bit_buf;
    int bits;

    i = 0;
    mismatch = ~coeff[0];

    bs.MakeLocalCopy(bit_buf,bits);

    NEEDBITS (bs,bit_buf, bits);

    while (1) {
      if (bit_buf >= 0x28000000) {

	tab = DCT_B14AC_5 - 5 + UBITS (bit_buf, 5);

	i += tab->run;
	if (i >= 64)
	  break;  // end of block

      normal_code:
	j = scan[i];
	bit_buf <<= tab->len;
	bits += tab->len + 1;
	if (quant_matrix) val = DequantizeIntra(tab->level , quantizer_scale , quant_matrix[j]);
	else val = tab->level;

	// if (bitstream_get (1)) val = -val;
	val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);

	Saturate (val);
	coeff[j] = val;
	mismatch ^= val;

	bit_buf <<= 1;
	NEEDBITS (bs,bit_buf, bits);

	continue;

      } else if (bit_buf >= 0x04000000) {

	tab = DCT_B14_8 - 4 + UBITS (bit_buf, 8);

	i += tab->run;
	if (i < 64)
	  goto normal_code;

	// escape code

	i += UBITS (bit_buf << 6, 6) - 64;
	if (i >= 64)
	  break;  // illegal, but check needed to avoid buffer overflow

	j = scan[i];

	DUMPBITS (bit_buf, bits, 12);
	NEEDBITS (bs,bit_buf, bits);
	if (quant_matrix)
	  {
	    val = DequantizeIntra(SBITS (bit_buf, 12) , quantizer_scale , quant_matrix[j]);
	  }
	else { val = SBITS(bit_buf,12); }

	Saturate(val);
	coeff[j] = val;
	mismatch ^= val;

	DUMPBITS (bit_buf, bits, 12);
	NEEDBITS (bs,bit_buf, bits);

	continue;

      } else if (bit_buf >= 0x02000000) {
	tab = DCT_B14_10 - 8 + UBITS (bit_buf, 10);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00800000) {
	tab = DCT_13 - 16 + UBITS (bit_buf, 13);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00200000) {
	tab = DCT_15 - 16 + UBITS (bit_buf, 15);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else {
	tab = DCT_16 + UBITS (bit_buf, 16);
	bit_buf <<= 16;
	bit_buf |= bs.GetNextWord () << (bits + 16);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      }
      break;  // illegal, but check needed to avoid buffer overflow
    }
    coeff[63] ^= mismatch & 1;
    DUMPBITS (bit_buf, bits, 2);        // dump end of block code
    bs.RestoreFromLocal(bit_buf,bits);
  }


  static void get_intra_block_B15(class BitReader_Fast& bs,short* coeff,const int* scan,
				  int quantizer_scale,
				  const uint8* quant_matrix)
  {
    int i;
    int j;
    int val;
    int mismatch;
    DCTtabb * tab;
    uint32 bit_buf;
    int bits;

    i = 0;
    mismatch = ~coeff[0];

    bs.MakeLocalCopy(bit_buf,bits);

    NEEDBITS (bs,bit_buf, bits);

    while (1) {
      if (bit_buf >= 0x04000000) {

	tab = DCT_B15_8 - 4 + UBITS (bit_buf, 8);

	i += tab->run;
	if (i < 64) {

	normal_code:
	  j = scan[i];
	  bit_buf <<= tab->len;
	  bits += tab->len + 1;
	  if (quant_matrix) val = DequantizeIntra(tab->level , quantizer_scale , quant_matrix[j]);
	  else val = tab->level;

	  // if (bitstream_get (1)) val = -val;
	  val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);

	  Saturate(val);
	  coeff[j] = val;
	  mismatch ^= val;

	  bit_buf <<= 1;
	  NEEDBITS (bs,bit_buf, bits);

	  continue;

	} else {

	  // end of block. I commented out this code because if we
	  // dont exit here we will still exit at the later test :)

	  //if (i >= 128) break;  // end of block

	  // escape code

	  i += UBITS (bit_buf << 6, 6) - 64;
	  if (i >= 64)
	    break;      // illegal, but check against buffer overflow

	  j = scan[i];

	  DUMPBITS (bit_buf, bits, 12);
	  NEEDBITS (bs,bit_buf, bits);
	  if (quant_matrix)
	    {
	      val = DequantizeIntra(SBITS (bit_buf, 12) , quantizer_scale , quant_matrix[j]);
	    }
	  else { val = SBITS(bit_buf,12); }

	  Saturate(val);
	  coeff[j] = val;
	  mismatch ^= val;

	  DUMPBITS (bit_buf, bits, 12);
	  NEEDBITS (bs,bit_buf, bits);

	  continue;

	}
      } else if (bit_buf >= 0x02000000) {
	tab = DCT_B15_10 - 8 + UBITS (bit_buf, 10);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00800000) {
	tab = DCT_13 - 16 + UBITS (bit_buf, 13);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00200000) {
	tab = DCT_15 - 16 + UBITS (bit_buf, 15);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else {
	tab = DCT_16 + UBITS (bit_buf, 16);
	bit_buf <<= 16;
	bit_buf |= bs.GetNextWord() << (bits + 16);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      }
      break;  // illegal, but check needed to avoid buffer overflow
    }
    coeff[63] ^= mismatch & 1;
    DUMPBITS (bit_buf, bits, 4);        // dump end of block code
    bs.RestoreFromLocal(bit_buf,bits);
  }


  static void get_non_intra_block(class BitReader_Fast& bs,short* coeff,const int* scan,
				  int quantizer_scale,
				  const uint8* quant_matrix)
  {
    int i;
    int j;
    int val;
    int mismatch;
    DCTtabb * tab;
    uint32 bit_buf;
    int bits;

    i = -1;
    mismatch = 1;

    bs.MakeLocalCopy(bit_buf,bits);

    NEEDBITS (bs,bit_buf, bits);
    if (bit_buf >= 0x28000000) {
      tab = DCT_B14DC_5 - 5 + UBITS (bit_buf, 5);
      goto entry_1;
    } else
      goto entry_2;

    while (1) {
      if (bit_buf >= 0x28000000) {

	tab = DCT_B14AC_5 - 5 + UBITS (bit_buf, 5);

      entry_1:
	i += tab->run;
	if (i >= 64)
	  break;  // end of block

      normal_code:
	j = scan[i];
	bit_buf <<= tab->len;
	bits += tab->len + 1;
	if (quant_matrix) val = DequantizeInter(tab->level , quantizer_scale , quant_matrix[j]);
	else val = tab->level;

	// if (bitstream_get (1)) val = -val;
	val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);

	Saturate(val);
	coeff[j] = val;
	mismatch ^= val;

	bit_buf <<= 1;
	NEEDBITS (bs,bit_buf, bits);

	continue;

      }

    entry_2:
      if (bit_buf >= 0x04000000) {

	tab = DCT_B14_8 - 4 + UBITS (bit_buf, 8);

	i += tab->run;
	if (i < 64)
	  goto normal_code;

	// escape code

	i += UBITS (bit_buf << 6, 6) - 64;
	if (i >= 64)
	  break;  // illegal, but check needed to avoid buffer overflow

	j = scan[i];

	DUMPBITS (bit_buf, bits, 12);
	NEEDBITS (bs,bit_buf, bits);
	if (quant_matrix)
	  {
	    val = DequantizeInter(SBITS (bit_buf, 12) + SBITS (bit_buf, 1) ,
				  quantizer_scale, quant_matrix[j]);
	  }
	else { val = SBITS(bit_buf,12); }

	Saturate(val);
	coeff[j] = val;
	mismatch ^= val;

	DUMPBITS (bit_buf, bits, 12);
	NEEDBITS (bs,bit_buf, bits);

	continue;

      } else if (bit_buf >= 0x02000000) {
	tab = DCT_B14_10 - 8 + UBITS (bit_buf, 10);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00800000) {
	tab = DCT_13 - 16 + UBITS (bit_buf, 13);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else if (bit_buf >= 0x00200000) {
	tab = DCT_15 - 16 + UBITS (bit_buf, 15);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      } else {
	tab = DCT_16 + UBITS (bit_buf, 16);
	bit_buf <<= 16;
	bit_buf |= bs.GetNextWord () << (bits + 16);
	i += tab->run;
	if (i < 64)
	  goto normal_code;
      }
      break;  // illegal, but check needed to avoid buffer overflow
    }
    coeff[63] ^= mismatch & 1;
    DUMPBITS (bit_buf, bits, 2);        // dump end of block code
    bs.RestoreFromLocal(bit_buf,bits);
  }




  MP2_DCTBlockDecoder::MP2_DCTBlockDecoder()
    : d_pichdr(NULL),
      d_on_the_fly_dequant(false),
      d_qscale(0),
      d_scan(NULL)
  {
  }


  void MP2_DCTBlockDecoder::SetPictureHeader(const MP2_PictureHeader& hdr)
  {
    d_pichdr = &hdr;
    d_scan = &scantab[d_pichdr->alternate_scan][0];
  }


  void MP2_DCTBlockDecoder::SetQCode(int q)
  {
    Assert(d_pichdr);

    d_qscale=AskQScaleTab(d_pichdr->q_scale_type)[q];

#if 0
    cout << "q: " << q << " -> ";
    for (int i=1;i<=31;i++)
      cout << ((int)AskQScaleTab(d_pichdr->q_scale_type)[i]) << " ";
    cout << endl;
#endif
  }


  int MP2_DCTBlockDecoder::DecodeIntra(DCTBlock_8x8& blk, BitReader_Fast& bs, bool lumi)
  {
    int new_dc_predictor;

    const uint8* matrix = NULL;
    if (d_on_the_fly_dequant)
      {
	if (lumi) matrix=&d_pichdr->seqhdr->intra_matrix.mat[0][0];
	else      matrix=&d_pichdr->seqhdr->chroma_intra_matrix.mat[0][0];
      }

    if (lumi)
      {
	int diff = get_luma_dc_dct_diff(bs);

	blk.coeff[0][0] += diff;
      }
    else
      {
	int diff = get_chroma_dc_dct_diff(bs);

	blk.coeff[0][0] += diff;
      }

    new_dc_predictor = blk.coeff[0][0];

    if (d_on_the_fly_dequant) blk.coeff[0][0] = (blk.coeff[0][0] << (11-d_pichdr->intra_dc_precision));


    if (d_pichdr->intra_vlc_format)
      {
	get_intra_block_B15(bs,&blk.coeff[0][0],d_scan,d_qscale,matrix);
      }
    else
      {
	if (d_pichdr->seqhdr->IsMPEG2)
	  {
	    get_intra_block_B14(bs,&blk.coeff[0][0],d_scan,d_qscale,matrix);
	  }
	else
	  {
	    get_mpeg1_intra_block(bs,&blk.coeff[0][0],d_scan,d_qscale,matrix);
	  }
      }

    return new_dc_predictor;
  }

  void MP2_DCTBlockDecoder::DecodeInter(DCTBlock_8x8& blk, BitReader_Fast& bs, bool lumi)
  {
    const uint8* matrix = NULL;
    if (d_on_the_fly_dequant)
      {
	if (lumi) matrix=&d_pichdr->seqhdr->nonintra_matrix.mat[0][0];
	else      matrix=&d_pichdr->seqhdr->chroma_nonintra_matrix.mat[0][0];
      }

    if (d_pichdr->seqhdr->IsMPEG2)
      {
	get_non_intra_block(bs,&blk.coeff[0][0],d_scan,d_qscale,matrix);
      }
    else
      {
	get_mpeg1_non_intra_block(bs,&blk.coeff[0][0],d_scan,d_qscale,matrix);
      }
  }

}
