/********************************************************************************
  mpeg2/syntax/pessource.hh
    Source for PES packets (& additional system headers).

  purpose:

  notes:

  to do:

  author(s):
   - Dirk Farin, dirk.farin@gmx.de

  modifications:
   03/Mar/2002 - Dirk Farin
     - new interface definition
 ********************************************************************************
    Copyright (C) 2002  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_PESSOURCE_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_PESSOURCE_HH

#include <libvideocoding/mpeg2/syntax/system.hh>

namespace videocoding {

  class MP2_PES_Source
  {
  public:
    MP2_PES_Source() { }
    virtual ~MP2_PES_Source() { }

    virtual void              Reset() { }
    virtual MP2_SystemPacket* GetNextPacket() = 0;
    virtual bool              IsEOF() const   = 0;
  };



  class MP2_ProgramStream_Decoder : public MP2_PES_Source
  {
  public:
    MP2_ProgramStream_Decoder() : d_istr(NULL) { }

    void              SetInputStream(InputStream&);
    MP2_SystemPacket* GetNextPacket();
    bool              IsEOF() const;

  private:
    InputStream* d_istr;
  };



  class MP2_ElementaryStream_Decoder : public MP2_PES_Source
  {
  public:
    MP2_ElementaryStream_Decoder();

    void              SetInputStream(InputStream&);
    void              SetStreamID(MP2_StartCode sc) { d_streamid=sc; }
    void              SetMinimalPacketLen(int len) { d_minimal_len=len; }

    MP2_SystemPacket* GetNextPacket();
    bool              IsEOF() const;

  private:
    int           d_minimal_len;
    InputStream*  d_istr;
    MP2_StartCode d_streamid;
  };


  class MP2_UniversalStream_Decoder : public MP2_PES_Source
  {
  public:
    MP2_UniversalStream_Decoder();

    /* initialization */

    void SetInputStream(InputStream&);
    void ForceStreamType(MP2_Stream_Type);
    void Reset() { }

    /* usage */

    MP2_SystemPacket* GetNextPacket();
    bool              IsEOF() const;

    MP2_Stream_Type   AskStreamType();

  private:
    InputStream* d_src;

    MP2_ElementaryStream_Decoder d_elem;
    MP2_ProgramStream_Decoder    d_prog;

    MP2_Stream_Type d_type;
    bool            d_forced_type;

    void DetermineType();
  };


  /* Only packets that start with a video start-code can pass.
   */
  class MP2_PES_JunkFilter : public MP2_PES_Source
  {
  public:
    MP2_PES_JunkFilter();

    void SetSource(MP2_PES_Source&);

    MP2_SystemPacket* GetNextPacket();
    bool              IsEOF() const;

  private:
    MP2_PES_Source* d_src;
  };

}

#endif
